#include <stdio.h>
#include "task.h"
#include "time_utils.h"

void set_priority(int prio) 
{
    struct sched_param param;
    param.sched_priority = prio;
    sched_setscheduler(pthread_self(), SCHED_FIFO, &param);
}

int isInMode(task_data_t *data) {
    int flag;
    pthread_mutex_lock(&data->modes->m);
    flag = data->modes->matrix[data->index][data->modes->curr_mode];
    pthread_mutex_unlock(&data->modes->m);
    return flag;
}

struct timespec task_disable(task_data_t *data)
{
    struct timespec now;
    sem_wait(&data->modes->block[data->index]);
    clock_gettime(CLOCK_REALTIME, &now);
    return now;
}

void *taskbody(void *arg)
{
    task_data_t data = *((task_data_t *)arg);
    struct timespec next;
    
    set_priority(data.priority);
    clock_gettime(CLOCK_REALTIME, &next);

    while (1) {
        if (!isInMode(&data)) next = task_disable(&data);
        printf("Task %d: sto eseguendo %d\n", data.index, data.period);
        timespec_add_us(&next, data.period);
        clock_nanosleep(CLOCK_REALTIME, 0, &next, 0);
    }
}

int check_event(task_data_t *data)
{
    return data->modes->dest_mode;
}

void activate_new_task(task_data_t *data, int n, int o)
{
    int i;
    for (i=0; i<NTASK; i++) {
        if (data->modes->matrix[data->index][o] == 0 &&
            data->modes->matrix[data->index][n] == 1) 
            sem_post(&data->modes->block[data->index]);
    }
}

void *taskmanager(void *arg)
{
    task_data_t data = *((task_data_t *)arg);
    struct timespec next;
    int old_mode;
    
    set_priority(data.priority);
    clock_gettime(CLOCK_REALTIME, &next);
    
    while(1) {
        pthread_mutex_lock(&data.modes->m);
        old_mode = data.modes->curr_mode;
        int new_mode = check_event(&data);
        if (old_mode != new_mode) {
            data.modes->curr_mode = new_mode;
            set_priority(1);
            activate_new_task(&data, new_mode, old_mode);
        }
        pthread_mutex_unlock(&data.modes->m);
                
        timespec_add_us(&next, data.period);
        clock_nanosleep(CLOCK_REALTIME, 0, &next, 0);
    }
}